@@ -75,8 +75,13 @@ void xdebug_execute_ex(zend_execute_data *execute_data TSRMLS_DC);
 void (*xdebug_old_execute_internal)(zend_execute_data *current_execute_data, zval *return_value);
 void xdebug_execute_internal(zend_execute_data *current_execute_data, zval *return_value);
 
+#if PHP_VERSION_ID >= 70300
+static int (*xdebug_orig_post_startup_cb)(void);
+static int xdebug_post_startup(void);
+#endif
+
 /* error callback replacement functions */
-void (*xdebug_old_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
+void (*xdebug_old_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args) ZEND_ATTRIBUTE_PTR_FORMAT(printf, 4, 0);
 void (*xdebug_new_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
 void xdebug_error_cb(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
 
@@ -394,7 +399,7 @@ PHP_INI_BEGIN()
 	STD_PHP_INI_ENTRY("xdebug.var_display_max_children", "128",         PHP_INI_ALL,    OnUpdateLong,   display_max_children, zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.var_display_max_data",     "512",         PHP_INI_ALL,    OnUpdateLong,   display_max_data,     zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.var_display_max_depth",    "3",           PHP_INI_ALL,    OnUpdateLong,   display_max_depth,    zend_xdebug_globals, xdebug_globals)
-	STD_PHP_INI_ENTRY("xdebug.cli_color",                "0",           PHP_INI_ALL,    OnUpdateBool,   cli_color,            zend_xdebug_globals, xdebug_globals)
+	STD_PHP_INI_ENTRY("xdebug.cli_color",                "0",           PHP_INI_ALL,    OnUpdateLong,   cli_color,            zend_xdebug_globals, xdebug_globals)
 
 	/* Scream support */
 	STD_PHP_INI_BOOLEAN("xdebug.scream",                 "0",           PHP_INI_ALL,    OnUpdateBool,   do_scream,            zend_xdebug_globals, xdebug_globals)
@@ -438,8 +443,12 @@ static void php_xdebug_init_globals (zend_xdebug_globals *xg TSRMLS_DC)
 	xg->context.do_next      = 0;
 	xg->context.do_finish    = 0;
 	xg->in_execution         = 0;
-	xg->remote_enabled       = 0;
-	xg->breakpoints_allowed  = 0;
+
+	xg->remote_connection_enabled = 0;
+	xg->remote_connection_pid     = 0;
+	xg->remote_log_file           = 0;
+	xg->breakpoints_allowed       = 0;
+
 	xg->profiler_enabled     = 0;
 	xg->do_monitor_functions = 0;
 
@@ -741,9 +750,12 @@ PHP_MINIT_FUNCTION(xdebug)
 	/* initialize aggregate call information hash */
 	zend_hash_init_ex(&XG(aggr_calls), 50, NULL, (dtor_func_t) xdebug_profile_aggr_call_entry_dtor, 1, 0);
 
-	/* Redirect compile and execute functions to our own */
+	/* Redirect compile and execute functions to our own. For PHP 7.3 and
+	 * later, we hook these in xdebug_post_startup instead */
+#if PHP_VERSION_ID < 70300
 	old_compile_file = zend_compile_file;
 	zend_compile_file = xdebug_compile_file;
+#endif
 
 	xdebug_old_execute_ex = zend_execute_ex;
 	zend_execute_ex = xdebug_execute_ex;
@@ -1066,12 +1078,8 @@ static void function_stack_entry_dtor(void *dummy, void *elem)
 	e->refcount--;
 
 	if (e->refcount == 0) {
-		if (e->function.function) {
-			xdfree(e->function.function);
-		}
-		if (e->function.class) {
-			xdfree(e->function.class);
-		}
+		xdebug_func_dtor_by_ref(&e->function);
+
 		if (e->filename) {
 			xdfree(e->filename);
 		}
@@ -1108,7 +1116,7 @@ static void function_stack_entry_dtor(void *dummy, void *elem)
 
 size_t xdebug_ub_write(const char *string, size_t length TSRMLS_DC)
 {
-	if (XG(remote_enabled)) {
+	if (xdebug_is_debug_connection_active_for_current_pid()) {
 		if (-1 == XG(context).handler->remote_stream_output(string, length TSRMLS_CC)) {
 			return 0;
 		}
@@ -1180,6 +1188,38 @@ PHP_RINIT_FUNCTION(xdebug)
 	ZEND_TSRMLS_CACHE_UPDATE();
 #endif
 
+#if PHP_VERSION_ID >= 70300 && PHP_VERSION_ID <= 70301
+	/* PHP Bug #77287 causes Xdebug to segfault if OPcache has the "compact
+	 * literals" optimisation turned on. So force it off for PHP 7.3.0 and
+	 * 7.3.1 */
+# if SIZEOF_ZEND_LONG == 4
+#  define XDEBUG_HEX8_FMT "0x%08" PRIX32
+# else
+#  define XDEBUG_HEX8_FMT "0x%08" PRIX64
+# endif
+	{
+		zend_long optimizer;
+		zend_string *key = zend_string_init(ZEND_STRL("opcache.optimization_level"), 1);
+		zend_string *value;
+
+		optimizer = INI_INT((char*) "opcache.optimization_level");
+		/* If it's empty, it relies on PHP's internal default, so we initialise
+		 * it to PHP's default in that case. */
+		if (!optimizer) {
+			optimizer = 0x7FFFBFFF;
+		}
+
+		optimizer &= ~(1<<10); /* DFA based optimization */
+
+		value = strpprintf(0, XDEBUG_HEX8_FMT, optimizer);
+
+		zend_alter_ini_entry(key, value, ZEND_INI_SYSTEM, ZEND_INI_STAGE_STARTUP);
+
+		zend_string_release(key);
+		zend_string_release(value);
+	}
+#endif
+
 	/* Get the ide key for this session */
 	XG(ide_key) = NULL;
 	idekey = xdebug_env_key(TSRMLS_C);
@@ -1262,9 +1302,10 @@ PHP_RINIT_FUNCTION(xdebug)
 		zend_throw_exception_hook = xdebug_throw_exception_hook;
 	}
 
-	XG(remote_enabled) = 0;
-	XG(profiler_enabled) = 0;
+	xdebug_mark_debug_connection_not_active();
 	XG(breakpoints_allowed) = 1;
+	XG(remote_log_file) = NULL;
+	XG(profiler_enabled) = 0;
 
 	/* Initialize some debugger context properties */
 	XG(context).program_name   = NULL;
@@ -1313,7 +1354,7 @@ ZEND_MODULE_POST_ZEND_DEACTIVATE_D(xdebug)
 {
 	TSRMLS_FETCH();
 
-	if (XG(remote_enabled)) {
+	if (XG(remote_connection_enabled)) {
 		XG(context).handler->remote_deinit(&(XG(context)));
 		xdebug_close_socket(XG(context).socket);
 	}
@@ -1430,6 +1471,21 @@ PHP_RSHUTDOWN_FUNCTION(xdebug)
 	return SUCCESS;
 }
 
+static int xdebug_info_printf(const char *fmt, ...) /* {{{ */
+{
+	char *buf;
+	size_t len, written;
+	va_list argv;
+
+	va_start(argv, fmt);
+	len = vspprintf(&buf, 0, fmt, argv);
+	va_end(argv);
+
+	written = php_output_write(buf, len);
+	efree(buf);
+	return written;
+}
+/* }}} */
 
 PHP_MINFO_FUNCTION(xdebug)
 {
@@ -1441,6 +1497,15 @@ PHP_MINFO_FUNCTION(xdebug)
 	php_info_print_table_row(2, "IDE Key", XG(ide_key));
 	php_info_print_table_end();
 
+	php_info_print_table_start();
+	if (!sapi_module.phpinfo_as_text) {
+		php_info_print_table_header(1, "Support Xdebug on Patreon");
+		xdebug_info_printf("<tr><td style='background-color: orangered; text-align: center'>%s</td></tr>\n", "<a style='font-size: large; color: white; background-color: transparent; font-weight: bold; text-decoration: underline' href='https://www.patreon.com/bePatron?u=7864328'>BECOME A PATRON</a>");
+	} else {
+		xdebug_info_printf("Support Xdebug on Patreon: https://www.patreon.com/bePatron?u=7864328\n");
+	}
+	php_info_print_table_end();
+
 	if (zend_xdebug_initialised == 0) {
 		php_info_print_table_start();
 		php_info_print_table_header(1, "XDEBUG NOT LOADED AS ZEND EXTENSION");
@@ -1597,7 +1662,7 @@ static void xdebug_throw_exception_hook(zval *exception TSRMLS_DC)
 	/* Start JIT if requested and not yet enabled */
 	xdebug_do_jit(TSRMLS_C);
 
-	if (XG(remote_enabled)) {
+	if (xdebug_is_debug_connection_active_for_current_pid()) {
 		int exception_breakpoint_found = 0;
 
 		/* Check if we have a wild card exception breakpoint */
@@ -1624,7 +1689,7 @@ static void xdebug_throw_exception_hook(zval *exception TSRMLS_DC)
 				code_str ? code_str : ((code && Z_TYPE_P(code) == IS_STRING) ? Z_STRVAL_P(code) : NULL),
 				Z_STRVAL_P(message))
 			) {
-				XG(remote_enabled) = 0;
+				xdebug_mark_debug_connection_not_active();
 			}
 		}
 	}
@@ -1686,9 +1751,7 @@ void xdebug_execute_ex(zend_execute_data *execute_data TSRMLS_DC)
 {
 	zend_op_array        *op_array = &(execute_data->func->op_array);
 	zend_execute_data    *edata = execute_data->prev_execute_data;
-	zval                 *dummy;
 	function_stack_entry *fse, *xfse;
-	char                 *magic_cookie = NULL;
 	int                   do_return = (XG(do_trace) && XG(trace_context));
 	int                   function_nr = 0;
 	xdebug_llist_element *le;
@@ -1727,108 +1790,61 @@ void xdebug_execute_ex(zend_execute_data *execute_data TSRMLS_DC)
 		XG(context).program_name = xdstrdup(STR_NAME_VAL(op_array->filename));
 	}
 
-	if (XG(level) == 0 && XG(in_execution)) {
-		/* Set session cookie if requested */
+	if (XG(in_execution)) {
+		/* Start debugger if this is the first main script, or previously a
+		 * connection was established and this process no longer has the same
+		 * PID */
 		if (
-			((
-				(dummy = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_GET]), "XDEBUG_SESSION_START", sizeof("XDEBUG_SESSION_START") - 1)) != NULL
-			) || (
-				(dummy = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_POST]), "XDEBUG_SESSION_START", sizeof("XDEBUG_SESSION_START") - 1)) != NULL
-			))
-			&& !SG(headers_sent)
+			XG(level) == 0 ||
+			(xdebug_is_debug_connection_active() && !xdebug_is_debug_connection_active_for_current_pid())
 		) {
-			convert_to_string_ex(dummy);
-			magic_cookie = xdstrdup(Z_STRVAL_P(dummy));
-			if (XG(ide_key)) {
-				xdfree(XG(ide_key));
-			}
-			XG(ide_key) = xdstrdup(magic_cookie);
-			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION"), Z_STRVAL_P(dummy), Z_STRLEN_P(dummy), time(NULL) + XG(remote_cookie_expire_time), "/", 1, NULL, 0, 0, 1, 0 TSRMLS_CC);
-		} else if (
-			(dummy = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_COOKIE]), "XDEBUG_SESSION", sizeof("XDEBUG_SESSION") - 1)) != NULL
-		) {
-			convert_to_string_ex(dummy);
-			magic_cookie = xdstrdup(Z_STRVAL_P(dummy));
-			if (XG(ide_key)) {
-				xdfree(XG(ide_key));
-			}
-			XG(ide_key) = xdstrdup(magic_cookie);
-		} else if (getenv("XDEBUG_CONFIG")) {
-			magic_cookie = xdstrdup(getenv("XDEBUG_CONFIG"));
-			if (XG(ide_key) && *XG(ide_key) && !SG(headers_sent)) {
-				xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION"), XG(ide_key), strlen(XG(ide_key)), time(NULL) + XG(remote_cookie_expire_time), "/", 1, NULL, 0, 0, 1, 0 TSRMLS_CC);
-			}
+			/* Start remote context if requested */
+			xdebug_do_req();
 		}
 
+		if (XG(level) == 0) {
+			/* Start profiler if requested, and we're in main script */
+			/* Check for special GET/POST parameter to start profiling */
+			if (
+				!XG(profiler_enabled) &&
+				(XG(profiler_enable) || xdebug_trigger_enabled(XG(profiler_enable_trigger), "XDEBUG_PROFILE", XG(profiler_enable_trigger_value) TSRMLS_CC))
+			) {
+				if (xdebug_profiler_init((char*) STR_NAME_VAL(op_array->filename) TSRMLS_CC) == SUCCESS) {
+					if (!SG(headers_sent)) {
+						sapi_header_line ctr = {0};
 
-		/* Remove session cookie if requested */
-		if (
-			((
-				zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_GET]), "XDEBUG_SESSION_STOP", sizeof("XDEBUG_SESSION_STOP") - 1) != NULL
-			) || (
-				zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_POST]), "XDEBUG_SESSION_STOP", sizeof("XDEBUG_SESSION_STOP") - 1) != NULL
-			))
-			&& !SG(headers_sent)
-		) {
-			if (magic_cookie) {
-				xdfree(magic_cookie);
-				magic_cookie = NULL;
-			}
-			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION"), (char*) "", 0, time(NULL) + XG(remote_cookie_expire_time), "/", 1, NULL, 0, 0, 1, 0 TSRMLS_CC);
-		}
+						ctr.line = xdebug_sprintf("X-Xdebug-Profile-Filename: %s", XG(profile_filename));
+						ctr.line_len = strlen(ctr.line);
+						sapi_header_op(SAPI_HEADER_REPLACE, &ctr);
+						xdfree(ctr.line);
+					}
 
-		/* Start remote context if requested */
-		if (
-			(magic_cookie || XG(remote_autostart)) &&
-			!XG(remote_enabled) &&
-			XG(remote_enable) &&
-			(XG(remote_mode) == XDEBUG_REQ)
-		) {
-			xdebug_init_debugger(TSRMLS_C);
-		}
-		if (magic_cookie) {
-			xdfree(magic_cookie);
-			magic_cookie = NULL;
-		}
-
-		/* Check for special GET/POST parameter to start profiling */
-		if (
-			!XG(profiler_enabled) &&
-			(XG(profiler_enable) || xdebug_trigger_enabled(XG(profiler_enable_trigger), "XDEBUG_PROFILE", XG(profiler_enable_trigger_value) TSRMLS_CC))
-		) {
-			if (xdebug_profiler_init((char*) STR_NAME_VAL(op_array->filename) TSRMLS_CC) == SUCCESS) {
-				if (!SG(headers_sent)) {
-					sapi_header_line ctr = {0};
-
-					ctr.line = xdebug_sprintf("X-Xdebug-Profile-Filename: %s", XG(profile_filename));
-					ctr.line_len = strlen(ctr.line);
-					sapi_header_op(SAPI_HEADER_REPLACE, &ctr);
-					xdfree(ctr.line);
+					XG(profiler_enabled) = 1;
 				}
-
-				XG(profiler_enabled) = 1;
 			}
-		}
 
-		if (
-			(XG(auto_trace) || xdebug_trigger_enabled(XG(trace_enable_trigger), "XDEBUG_TRACE", XG(trace_enable_trigger_value) TSRMLS_CC))
-			&& XG(trace_output_dir) && strlen(XG(trace_output_dir))
-		) {
-			/* In case we do an auto-trace we are not interested in the return
-			 * value, but we still have to free it. */
-			xdfree(xdebug_start_trace(NULL, STR_NAME_VAL(op_array->filename), XG(trace_options) TSRMLS_CC));
-		}
+			/* Start auto-tracer if requested, and we're in main script */
+			if (
+				(XG(auto_trace) || xdebug_trigger_enabled(XG(trace_enable_trigger), "XDEBUG_TRACE", XG(trace_enable_trigger_value) TSRMLS_CC))
+				&& XG(trace_output_dir) && strlen(XG(trace_output_dir))
+			) {
+				/* In case we do an auto-trace we are not interested in the return
+				 * value, but we still have to free it. */
+				xdfree(xdebug_start_trace(NULL, STR_NAME_VAL(op_array->filename), XG(trace_options) TSRMLS_CC));
+			}
 
-		if (!XG(gc_stats_enabled) && XG(gc_stats_enable)) {
-			if (xdebug_gc_stats_init(NULL, STR_NAME_VAL(op_array->filename)) == SUCCESS) {
-				XG(gc_stats_enabled) = 1;
+			/* Start GC stats collector if requested, and we're in main script */
+			if (!XG(gc_stats_enabled) && XG(gc_stats_enable)) {
+				if (xdebug_gc_stats_init(NULL, STR_NAME_VAL(op_array->filename)) == SUCCESS) {
+					XG(gc_stats_enabled) = 1;
+				}
 			}
 		}
 	}
 
 	XG(level)++;
 	if ((signed long) XG(level) > XG(max_nesting_level) && (XG(max_nesting_level) != -1)) {
-		zend_throw_exception_ex(zend_ce_error, 0, "Maximum function nesting level of '%ld' reached, aborting!", XG(max_nesting_level));
+		zend_throw_exception_ex(zend_ce_error, 0, "Maximum function nesting level of '" ZEND_LONG_FMT "' reached, aborting!", XG(max_nesting_level));
 	}
 
 	fse = xdebug_add_stack_frame(edata, op_array, XDEBUG_EXTERNAL TSRMLS_CC);
@@ -1858,7 +1874,7 @@ void xdebug_execute_ex(zend_execute_data *execute_data TSRMLS_DC)
 		fse->This = NULL;
 	}
 
-	if (XG(stack) && (XG(remote_enabled) || XG(collect_vars) || XG(show_local_vars))) {
+	if (XG(stack) && (XG(collect_vars) || XG(show_local_vars) || xdebug_is_debug_connection_active_for_current_pid())) {
 		/* Because include/require is treated as a stack level, we have to add used
 		 * variables in include/required files to all the stack levels above, until
 		 * we hit a function or the top level stack.  This is so that the variables
@@ -1892,14 +1908,14 @@ void xdebug_execute_ex(zend_execute_data *execute_data TSRMLS_DC)
 	/* If we're in an eval, we need to create an ID for it. This ID however
 	 * depends on the debugger mechanism in use so we need to call a function
 	 * in the handler for it */
-	if (XG(remote_enabled) && XG(context).handler->register_eval_id && fse->function.type == XFUNC_EVAL) {
+	if (fse->function.type == XFUNC_EVAL && xdebug_is_debug_connection_active_for_current_pid() && XG(context).handler->register_eval_id) {
 		XG(context).handler->register_eval_id(&(XG(context)), fse);
 	}
 
 	/* Check for entry breakpoints */
-	if (XG(remote_enabled) && XG(breakpoints_allowed)) {
+	if (xdebug_is_debug_connection_active_for_current_pid() && XG(breakpoints_allowed)) {
 		if (!handle_breakpoints(fse, XDEBUG_BRK_FUNC_CALL)) {
-			XG(remote_enabled) = 0;
+			xdebug_mark_debug_connection_not_active();
 		}
 	}
 
@@ -1944,9 +1960,9 @@ void xdebug_execute_ex(zend_execute_data *execute_data TSRMLS_DC)
 	}
 
 	/* Check for return breakpoints */
-	if (XG(remote_enabled) && XG(breakpoints_allowed)) {
+	if (xdebug_is_debug_connection_active_for_current_pid() && XG(breakpoints_allowed)) {
 		if (!handle_breakpoints(fse, XDEBUG_BRK_FUNC_RETURN)) {
-			XG(remote_enabled) = 0;
+			xdebug_mark_debug_connection_not_active();
 		}
 	}
 
@@ -1958,16 +1974,29 @@ void xdebug_execute_ex(zend_execute_data *execute_data TSRMLS_DC)
 	XG(level)--;
 }
 
-static int check_soap_call(function_stack_entry *fse)
+static int check_soap_call(function_stack_entry *fse, zend_execute_data *execute_data)
 {
-	if (fse->function.class &&
-		(
-			(strstr(fse->function.class, "SoapClient") != NULL) ||
-			(strstr(fse->function.class, "SoapServer") != NULL)
-		) &&
+	if (
+		fse->function.class &&
+		Z_OBJ(EX(This)) &&
+		Z_TYPE(EX(This)) == IS_OBJECT &&
 		(zend_hash_str_find_ptr(&module_registry, "soap", sizeof("soap") - 1) != NULL)
 	) {
-		return 1;
+		zend_class_entry *soap_server_ce, *soap_client_ce;
+
+		soap_server_ce = zend_hash_str_find_ptr(CG(class_table), "soapserver", 10);
+		soap_client_ce = zend_hash_str_find_ptr(CG(class_table), "soapclient", 10);
+
+		if (!soap_server_ce || !soap_client_ce) {
+			return 0;
+		}
+
+		if (
+			(instanceof_function(Z_OBJCE(EX(This)), soap_server_ce)) ||
+			(instanceof_function(Z_OBJCE(EX(This)), soap_client_ce))
+		) {
+			return 1;
+		}
 	}
 	return 0;
 }
@@ -1980,11 +2009,11 @@ void xdebug_execute_internal(zend_execute_data *current_execute_data, zval *retu
 	int                   function_nr = 0;
 
 	int                   restore_error_handler_situation = 0;
-	void                (*tmp_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args) = NULL;
+	void                (*tmp_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args) ZEND_ATTRIBUTE_PTR_FORMAT(printf, 4, 0) = NULL;
 
 	XG(level)++;
 	if ((signed long) XG(level) > XG(max_nesting_level) && (XG(max_nesting_level) != -1)) {
-		zend_throw_exception_ex(zend_ce_error, 0, "Maximum function nesting level of '%ld' reached, aborting!", XG(max_nesting_level));
+		zend_throw_exception_ex(zend_ce_error, 0, "Maximum function nesting level of '" ZEND_LONG_FMT "' reached, aborting!", XG(max_nesting_level));
 	}
 
 	fse = xdebug_add_stack_frame(edata, &edata->func->op_array, XDEBUG_INTERNAL TSRMLS_CC);
@@ -1997,14 +2026,14 @@ void xdebug_execute_internal(zend_execute_data *current_execute_data, zval *retu
 	}
 
 	/* Check for entry breakpoints */
-	if (XG(remote_enabled) && XG(breakpoints_allowed)) {
+	if (xdebug_is_debug_connection_active_for_current_pid() && XG(breakpoints_allowed)) {
 		if (!handle_breakpoints(fse, XDEBUG_BRK_FUNC_CALL)) {
-			XG(remote_enabled) = 0;
+			xdebug_mark_debug_connection_not_active();
 		}
 	}
 
 	/* Check for SOAP */
-	if (check_soap_call(fse)) {
+	if (check_soap_call(fse, current_execute_data)) {
 		restore_error_handler_situation = 1;
 		tmp_error_cb = zend_error_cb;
 		zend_error_cb = xdebug_old_error_cb;
@@ -2041,9 +2070,9 @@ void xdebug_execute_internal(zend_execute_data *current_execute_data, zval *retu
 	}
 
 	/* Check for return breakpoints */
-	if (XG(remote_enabled) && XG(breakpoints_allowed)) {
+	if (xdebug_is_debug_connection_active_for_current_pid() && XG(breakpoints_allowed)) {
 		if (!handle_breakpoints(fse, XDEBUG_BRK_FUNC_RETURN)) {
-			XG(remote_enabled) = 0;
+			xdebug_mark_debug_connection_not_active();
 		}
 	}
 
@@ -2137,7 +2166,7 @@ static int xdebug_header_handler(sapi_header_struct *h, sapi_header_op_enum op,
    Dummy function to prevent time limit from being set within the script */
 PHP_FUNCTION(xdebug_set_time_limit)
 {
-	if (!XG(remote_enabled)) {
+	if (!xdebug_is_debug_connection_active_for_current_pid()) {
 		XG(orig_set_time_limit_func)(INTERNAL_FUNCTION_PARAM_PASSTHRU);
 	}
 }
@@ -2191,17 +2220,17 @@ PHP_FUNCTION(xdebug_var_dump)
 		else if (PG(html_errors)) {
 			val = xdebug_get_zval_value_fancy(NULL, (zval*) &args[i], 0, NULL);
 			PHPWRITE(val->d, val->l);
-			xdfree(val);
+			xdebug_str_free(val);
 		}
 		else if ((XG(cli_color) == 1 && xdebug_is_output_tty(TSRMLS_C)) || (XG(cli_color) == 2)) {
 			val = xdebug_get_zval_value_ansi((zval*) &args[i], 0, NULL);
 			PHPWRITE(val->d, val->l);
-			xdfree(val);
+			xdebug_str_free(val);
 		}
 		else {
 			val = xdebug_get_zval_value_text((zval*) &args[i], 0, NULL);
 			PHPWRITE(val->d, val->l);
-			xdfree(val);
+			xdebug_str_free(val);
 		}
 	}
 
@@ -2357,7 +2386,7 @@ PHP_FUNCTION(xdebug_is_enabled)
 
 PHP_FUNCTION(xdebug_is_debugger_active)
 {
-	RETURN_BOOL(XG(remote_enabled));
+	RETURN_BOOL(xdebug_is_debug_connection_active_for_current_pid());
 }
 
 PHP_FUNCTION(xdebug_break)
@@ -2505,13 +2534,13 @@ ZEND_DLEXPORT void xdebug_statement_call(zend_op_array *op_array)
 		xdebug_count_line(file, lineno, 0, 0 TSRMLS_CC);
 	}
 
-	if (XG(remote_enabled)) {
+	if (xdebug_is_debug_connection_active_for_current_pid()) {
 
 		if (XG(context).do_break) {
 			XG(context).do_break = 0;
 
 			if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), file, lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
-				XG(remote_enabled) = 0;
+				xdebug_mark_debug_connection_not_active();
 				return;
 			}
 		}
@@ -2538,7 +2567,7 @@ ZEND_DLEXPORT void xdebug_statement_call(zend_op_array *op_array)
 			XG(context).do_finish = 0;
 
 			if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), file, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
-				XG(remote_enabled) = 0;
+				xdebug_mark_debug_connection_not_active();
 				return;
 			}
 			return;
@@ -2549,7 +2578,7 @@ ZEND_DLEXPORT void xdebug_statement_call(zend_op_array *op_array)
 			XG(context).do_next = 0;
 
 			if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), file, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
-				XG(remote_enabled) = 0;
+				xdebug_mark_debug_connection_not_active();
 				return;
 			}
 			return;
@@ -2560,7 +2589,7 @@ ZEND_DLEXPORT void xdebug_statement_call(zend_op_array *op_array)
 			XG(context).do_step = 0;
 
 			if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), file, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
-				XG(remote_enabled) = 0;
+				xdebug_mark_debug_connection_not_active();
 				return;
 			}
 			return;
@@ -2606,7 +2635,7 @@ ZEND_DLEXPORT void xdebug_statement_call(zend_op_array *op_array)
 					}
 					if (break_ok && xdebug_handle_hit_value(extra_brk_info)) {
 						if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), file, lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
-							XG(remote_enabled) = 0;
+							xdebug_mark_debug_connection_not_active();
 							break;
 						}
 						return;
@@ -2650,9 +2679,33 @@ ZEND_DLEXPORT int xdebug_zend_startup(zend_extension *extension)
 
 	zend_xdebug_initialised = 1;
 
+#if PHP_VERSION_ID >= 70300
+	xdebug_orig_post_startup_cb = zend_post_startup_cb;
+	zend_post_startup_cb = xdebug_post_startup;
+
 	return zend_startup_module(&xdebug_module_entry);
 }
 
+static int xdebug_post_startup(void)
+{
+	if (xdebug_orig_post_startup_cb) {
+		int (*cb)(void) = xdebug_orig_post_startup_cb;
+
+		xdebug_orig_post_startup_cb = NULL;
+		if (cb() != SUCCESS) {
+			return FAILURE;
+		}
+	}
+
+	old_compile_file = zend_compile_file;
+	zend_compile_file = xdebug_compile_file;
+	
+	return SUCCESS;
+#else
+	return zend_startup_module(&xdebug_module_entry);
+#endif
+}
+
 ZEND_DLEXPORT void xdebug_zend_shutdown(zend_extension *extension)
 {
 	/* Remove our hooks to output handlers (header and output writer) */
